package cn.iocoder.yudao.module.system.service.worklog;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogFormDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogRuleDO;
import cn.iocoder.yudao.module.system.dal.mysql.worklog.LogInstanceMapper;
import cn.iocoder.yudao.module.system.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogReadUserRespDTO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.WeekFields;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_INSTANCE_NOT_EXISTS;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_RULE_NOT_FIND;

/**
 * 日志实例的拓展 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LogInstanceServiceImpl implements LogInstanceService {

    @Resource
    private LogInstanceMapper logInstanceMapper;

    @Resource
    private DeptService deptService;

    @Resource
    private LogRuleService logRuleService;

    @Resource
    private LogFormService logFormService;

    @Resource
    private AdminUserService adminUserService;

    @Resource
    private LogReadService logReadService;

    @Override
    public Long createLogInstance(LogInstanceSaveReqVO createReqVO) {

        //校验当前日志类型的日志 是否已提交过
        validateLogInstanceByType(createReqVO);

        LogInstanceDO logInstance = BeanUtils.toBean(createReqVO, LogInstanceDO.class);

        //获取日志类型
        LogFormDO logFormDo = logFormService.getForm(createReqVO.getFormId());
        LogRuleDO logRuleDO = logRuleService.getLogRule(logFormDo.getRuleId());
        if (logRuleDO == null) {
            throw exception(LOG_RULE_NOT_FIND);
        }

        //获取部门信息
        AdminUserDO adminUserDO = adminUserService.getUser(getLoginUserId());

        //设置发起人用户编号
        logInstance.setStartUserId(getLoginUserId());
        //设置发起人部门编号
        logInstance.setDeptId(adminUserDO.getDeptId());
        //设置发起人用户名称
        logInstance.setStartUserName(adminUserDO.getNickname());
        //设置发起人用户头像
        logInstance.setAvatar(adminUserDO.getAvatar());
        //设置日志类型
        logInstance.setType(logRuleDO.getType());
        //设置日志名称
        logInstance.setName(adminUserDO.getNickname() + "的" + logFormDo.getName());
        //设置时间
        logInstance.setTime(DateUtil.format(new Date(),"yyyy-MM-dd"));

        // 插入
        logInstanceMapper.insert(logInstance);

        //创建日志时，查询可以查看发起人日志的用户组  用线程控制
        new Thread(() -> {

            List<LogReadUserRespDTO> respDTOS = logInstanceMapper.selectRaedUser(adminUserDO.getDeptId());
            logReadService.createLogRule(respDTOS, logInstance.getId(), logInstance.getStartUserId());
        }).start();

        // 返回
        return logInstance.getId();
    }

    private void validateLogInstanceByType(LogInstanceSaveReqVO createReqVO) {

        LogInstanceDO logInstanceDO = getUpLog(createReqVO.getType());
        if (logInstanceDO != null) {

            LocalDate oldTime = logInstanceDO.getCreateTime().toLocalDate();
            LocalDate nowTime = LocalDate.now();

            //根据日志类型判断
            switch (createReqVO.getType()){

                //日志类型为日报
                case 1:
                    if (oldTime.equals(nowTime)) {

                        throw exception(ErrorCodeConstants.LOG_TYPE_EXISTS, createReqVO.getType());
                    }
                //日志类型为周报
                case 2:
                    WeekFields weekFields = WeekFields.of(Locale.getDefault());

                    // 获取两个日期所属的周数和年份
                    int oldWeek = oldTime.get(weekFields.weekOfWeekBasedYear());
                    int oldYear = oldTime.get(weekFields.weekBasedYear());

                    int nowWeek = nowTime.get(weekFields.weekOfWeekBasedYear());
                    int nowYear = nowTime.get(weekFields.weekBasedYear());

                    if (oldWeek == nowWeek && oldYear == nowYear) {

                        throw exception(ErrorCodeConstants.LOG_TYPE_EXISTS, createReqVO.getType());
                    }
                //日志类型为月报
                case 3:
                    if (oldTime.getYear() == nowTime.getYear() && oldTime.getMonth() == nowTime.getMonth()) {

                        throw exception(ErrorCodeConstants.LOG_TYPE_EXISTS, createReqVO.getType());
                    }
                //日志类型为年报
                case 4:
                    if (oldTime.getYear() == nowTime.getYear()) {

                        throw exception(ErrorCodeConstants.LOG_TYPE_EXISTS, createReqVO.getType());
                    }
            }
        }
    }

    @Override
    public void updateLogInstance(LogInstanceSaveReqVO updateReqVO) {
        // 校验存在
        validateLogInstanceExists(updateReqVO.getId());
        // 更新
        LogInstanceDO updateObj = BeanUtils.toBean(updateReqVO, LogInstanceDO.class);
        logInstanceMapper.updateById(updateObj);
    }

    @Override
    public void deleteLogInstance(Long id) {
        // 校验存在
        validateLogInstanceExists(id);
        // 删除
        logInstanceMapper.deleteById(id);
    }

    private void validateLogInstanceExists(Long id) {
        if (logInstanceMapper.selectById(id) == null) {
            throw exception(LOG_INSTANCE_NOT_EXISTS);
        }
    }

    @Override
    public LogInstanceDO getLogInstance(Long id) {
        return logInstanceMapper.selectById(id);
    }

    @Override
    public IPage<LogInstanceRespVO> getLogInstancePage(LogInstancePageReqVO pageReqVO) {

        Page<LogInstanceRespVO> page = new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize());
        IPage<LogInstanceRespVO> pageList = logInstanceMapper.selectPageResult(page, pageReqVO, getLoginUserId());

        List<LogInstanceRespVO> records = pageList.getRecords();
        if (!records.isEmpty()) {

            //模版ids过滤
            List<Long> workFormIds = records.stream().map(LogInstanceRespVO::getFormId).collect(Collectors.toList());

            //  查询模版列表
            List<LogFormDO> formList = logFormService.getFormList(workFormIds);
            Map<Long, LogFormDO> formMap = formList.stream().collect(Collectors.toMap(LogFormDO::getId, item -> item));

            //遍历
            records.forEach(item -> {

                //设置日志内部分
                LogFormDO logFormDO = formMap.get(item.getFormId());
                List<String> fields = logFormDO.getFields();
                StringBuilder workLogContent = new StringBuilder();
                if (!item.getFormVariables().isEmpty()) {
                    JSONObject workLogContentJson = new JSONObject(item.getFormVariables());
                    for (String fieldItem : fields) {
                        JSONObject fieldJson = new JSONObject(fieldItem);
                        String fieldStr = fieldJson.getStr("field");
                        String title = fieldJson.getStr("title");
                        String field = workLogContentJson.getStr(fieldStr);
                        workLogContent.append(title).append(":").append(field);
                    }
                }
                item.setWorkLogContent(workLogContent.toString());

                //设置时间
                LocalDate nowDate = LocalDate.now();
                LocalDateTime createTime = LocalDateTime.parse(item.getCreateTime(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                //日志创建时间是当前日期时
                if (createTime.toLocalDate().equals(nowDate)) {
                    item.setCreateTime("今日 " + createTime.getHour() + ":" + createTime.getMinute());
                }
                //日志创建时间是当前日期前一天时
                if (createTime.toLocalDate().equals(nowDate.minusDays(1))) {
                    item.setCreateTime("昨日 " + createTime.getHour() + ":" + createTime.getMinute());
                }
            });
        }

        return pageList;
    }

    @Override
    public List<DeptRespVO> getDeptInfo() {

        List<LogInstanceDO> logInstanceDOS = logInstanceMapper.selectList();
        List<Long> deptIds = logInstanceDOS.stream()
                .map(LogInstanceDO::getDeptId) // 获取每个LogInstanceDO的 deptId
                .collect(Collectors.toList()); // 收集成List

        List<DeptDO> deptDOS = deptService.getDeptList(deptIds);
        return BeanUtils.toBean(deptDOS, DeptRespVO.class);
    }

    @Override
    public LogInstanceDO getUpLog(Integer type) {

        LogInstanceDO logInstanceDO = new LogInstanceDO();

        List<LogInstanceDO> logInstanceDOS = logInstanceMapper.getUpLog(getLoginUserId(), type);
        if (!logInstanceDOS.isEmpty()) {
            logInstanceDO = logInstanceDOS.get(0);
        }
        return logInstanceDO;
    }
}